@@ -43,13 +43,25 @@ knowledge of the CeCILL license and that you accept its terms.
 
 #include <QApplication>
 #include <QtGui>
+#include <QVBoxLayout>
+#include <QFormLayout>
+#include <QPushButton>
+#include <QMessageBox>
+#include <QButtonGroup>
+#include <QDockWidget>
+#include <QUndoView>
+#include <QLineEdit>
+#include <QAction>
+#include <QMenuBar>
+#include <QInputDialog>
+#include <QGraphicsTextItem>
 
 #include <Bpp/Qt/QtGraphicDevice.h>
 
 #include <Bpp/Numeric/DataTable.h>
 
 #include <Bpp/Phyl/Tree.h>
-#include <Bpp/Phyl/Io.all>
+#include <Bpp/Phyl/Io/Nhx.h>
 #include <Bpp/Phyl/Graphics/PhylogramPlot.h>
 
 #include <fstream>
@@ -79,8 +91,9 @@ TranslateNameChooser::TranslateNameChooser(PhyView* phyview) :
   fileFilters_ << "Coma separated columns (*.txt *.csv)"
                << "Tab separated columns (*.txt *.csv)";
   fileDialog_->setNameFilters(fileFilters_);
+  fileDialog_->setOptions(QFileDialog::DontUseNativeDialog);
   hasHeader_ = new QCheckBox(tr("File has header line"));
-  QGridLayout *dlayout = dynamic_cast<QGridLayout*>(fileDialog_->layout());
+  QGridLayout *dlayout = dynamic_cast<QGridLayout*>(fileDialog_->layout()); //Check that here!!
   dlayout->addWidget(hasHeader_, 4, 0);
   QFormLayout* layout = new QFormLayout;
   fromList_ = new QComboBox;
@@ -104,26 +117,30 @@ void TranslateNameChooser::translateTree(TreeTemplate<Node>& tree)
     if (fileDialog_->selectedNameFilter() == fileFilters_[1])
       sep = "\t";
     ifstream file(path[0].toStdString().c_str(), ios::in);
-    DataTable* table = DataTable::read(file, sep, hasHeader_->isChecked());
+    try {
+      DataTable* table = DataTable::read(file, sep, hasHeader_->isChecked());
 
-    //Clean button groups:
-    fromList_->clear();
-    toList_->clear();
+      //Clean button groups:
+      fromList_->clear();
+      toList_->clear();
 
-    //Now add the new ones:
-    if (!hasHeader_->isChecked()) {
-      vector<string> names;
-      for (unsigned int i = 0; i < table->getNumberOfColumns(); ++i) {
-        names.push_back("Col" + TextTools::toString(i + 1));
+      //Now add the new ones:
+      if (!hasHeader_->isChecked()) {
+        vector<string> names;
+        for (unsigned int i = 0; i < table->getNumberOfColumns(); ++i) {
+          names.push_back("Col" + TextTools::toString(i + 1));
+        }
+        table->setColumnNames(names);
       }
-      table->setColumnNames(names);
+      for (unsigned int i = 0; i < table->getNumberOfColumns(); ++i) {
+        fromList_->addItem(QtTools::toQt(table->getColumnName(i)));
+          toList_->addItem(QtTools::toQt(table->getColumnName(i)));
+      }
+      if (exec() == QDialog::Accepted)
+        phyview_->submitCommand(new TranslateNodeNamesCommand(phyview_->getActiveDocument(), *table, fromList_->currentIndex(),  toList_->currentIndex()));
+    } catch (Exception& e) {
+      QMessageBox::critical(this, tr("Ouch..."), tr("Error when reading table:\n") + tr(e.what()));
     }
-    for (unsigned int i = 0; i < table->getNumberOfColumns(); ++i) {
-      fromList_->addItem(QtTools::toQt(table->getColumnName(i)));
-        toList_->addItem(QtTools::toQt(table->getColumnName(i)));
-    }
-    if (exec() == QDialog::Accepted)
-      phyview_->submitCommand(new TranslateNodeNamesCommand(phyview_->getActiveDocument(), *table, fromList_->currentIndex(),  toList_->currentIndex()));
   }
 }
 
@@ -311,7 +328,13 @@ void MouseActionListener::mousePressEvent(QMouseEvent *event)
       phyview_->submitCommand(new OrderCommand(phyview_->getActiveDocument(), nodeId, false));
     }
     else if (action == "Root on node")
-      phyview_->submitCommand(new RerootCommand(phyview_->getActiveDocument(), nodeId));
+    {
+      if (phyview_->getActiveDocument()->getTree()->getNode(nodeId)->isLeaf()) {
+        QMessageBox::warning(phyview_, "PhyView", "Cannot root on a leaf.", QMessageBox::Cancel);
+      } else {     
+        phyview_->submitCommand(new RerootCommand(phyview_->getActiveDocument(), nodeId));
+      }
+    }
     else if (action == "Root on branch")
       phyview_->submitCommand(new OutgroupCommand(phyview_->getActiveDocument(), nodeId));
     else if (action == "Collapse") {
@@ -331,32 +354,24 @@ void MouseActionListener::mousePressEvent(QMouseEvent *event)
     }
     else if (action == "Copy subtree") {
       Node* subtree = TreeTemplateTools::cloneSubtree<Node>(*phyview_->getActiveDocument()->getTree()->getNode(nodeId));
-      auto_ptr< TreeTemplate<Node> > tt(new TreeTemplate<Node>(subtree));
+      unique_ptr< TreeTemplate<Node> > tt(new TreeTemplate<Node>(subtree));
       phyview_->createNewDocument(tt.get());
     }
     else if (action == "Cut subtree") {
       Node* subtree = TreeTemplateTools::cloneSubtree<Node>(*phyview_->getActiveDocument()->getTree()->getNode(nodeId));
-      auto_ptr< TreeTemplate<Node> > tt(new TreeTemplate<Node>(subtree));
+      unique_ptr< TreeTemplate<Node> > tt(new TreeTemplate<Node>(subtree));
       phyview_->submitCommand(new DeleteSubtreeCommand(phyview_->getActiveDocument(), nodeId));
       phyview_->createNewDocument(tt.get());
     }
     else if (action == "Insert on node") {
-      if (phyview_->getNonActiveDocuments().size() == 0) {
-        QMessageBox::critical(phyview_, QString("Oups..."), QString("No tree to insert."));
-        return;
-      }
-      TreeTemplate<Node>* tree = pickTree_();
+      TreeTemplate<Node>* tree = phyview_->pickTree();
       if (tree) {
         Node* subtree = TreeTemplateTools::cloneSubtree<Node>(*tree->getRootNode());
         phyview_->submitCommand(new InsertSubtreeAtNodeCommand(phyview_->getActiveDocument(), nodeId, subtree));
       }
     }
     else if (action == "Insert on branch") {
-      if (phyview_->getNonActiveDocuments().size() == 0) {
-        QMessageBox::critical(phyview_, QString("Oups..."), QString("No tree to insert."));
-        return;
-      }
-      TreeTemplate<Node>* tree = pickTree_();
+      TreeTemplate<Node>* tree = phyview_->pickTree();
       if (tree) {
         Node* subtree = TreeTemplateTools::cloneSubtree<Node>(*tree->getRootNode());
         phyview_->submitCommand(new InsertSubtreeOnBranchCommand(phyview_->getActiveDocument(), nodeId, subtree));
@@ -367,28 +382,6 @@ void MouseActionListener::mousePressEvent(QMouseEvent *event)
 
 
 
-TreeTemplate<Node>* MouseActionListener::pickTree_()
-{
-  QList<TreeDocument*> documents = phyview_->getNonActiveDocuments();
-  treeList_->clear();
-  for (int i = 0; i < documents.size(); ++i) {
-    QString text = QtTools::toQt(documents[i]->getName());
-    if (text == "") text = "(unknown)";
-    vector<string> leaves = documents[i]->getTree()->getLeavesNames(); 
-    text += QtTools::toQt(" " + TextTools::toString(leaves.size()) + " leaves ");
-    for (unsigned int j = 0; j < min(static_cast<unsigned int>(leaves.size()), 5u); ++j) {
-      text += QtTools::toQt(", " + leaves[j]);
-    }
-    if (leaves.size() >= 5) text += "...";
-    treeList_->addItem(text);
-  }
-  treeChooser_->exec();
-  return documents[treeList_->currentRow()]->getTree();
-}
-
-
-
-
 PhyView::PhyView():
   manager_(),
   collapsedNodesListener_(true)
@@ -410,6 +403,13 @@ void PhyView::initGui_()
   connect(mdiArea_, SIGNAL(subWindowActivated(QMdiSubWindow*)), this, SLOT(setCurrentSubWindow(QMdiSubWindow*)));
   setCentralWidget(mdiArea_);
   
+  //Trees panel:
+  createTreesPanel_();
+  treesDockWidget_ = new QDockWidget(tr("Trees"));
+  treesDockWidget_->setWidget(treesPanel_);
+  treesDockWidget_->setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
+  addDockWidget(Qt::LeftDockWidgetArea, treesDockWidget_);
+
   //Stats panel:
   createStatsPanel_();
   statsDockWidget_ = new QDockWidget(tr("Statistics"));
@@ -517,6 +517,23 @@ void PhyView::createDisplayPanel_()
   displayPanel_->setLayout(layout);
 }
 
+void PhyView::createTreesPanel_()
+{
+  treesPanel_ = new QWidget(this);
+  QVBoxLayout* treesLayout = new QVBoxLayout;
+  treesTable_ = new QTableWidget;
+  treesTable_->setColumnCount(2);
+  treesTable_->setHorizontalHeaderLabels(QString("Tree;Size").split(";"));
+  treesTable_->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
+  treesTable_->setEditTriggers(QAbstractItemView::NoEditTriggers);
+  treesTable_->setSelectionBehavior(QAbstractItemView::SelectRows);
+  treesTable_->setSelectionMode(QAbstractItemView::SingleSelection);
+  connect(treesTable_, SIGNAL(itemClicked(QTableWidgetItem*)), this, SLOT(activateSelectedDocument()));
+  treesLayout->addWidget(treesTable_);
+  treesLayout->addStretch(1);
+  treesPanel_->setLayout(treesLayout);
+}
+
 void PhyView::createStatsPanel_()
 {
   statsPanel_ = new QWidget(this);
@@ -551,6 +568,11 @@ void PhyView::createBrlenPanel_()
   
   brlenLayout->addWidget(brlenSetLengthsBox);
 
+  //Remove all branch lengths:
+  QPushButton* brlenRemoveAll = new QPushButton(tr("Remove all lengths"));
+  connect(brlenRemoveAll, SIGNAL(clicked(bool)), this, SLOT(deleteAllLengths()));
+  brlenLayout->addWidget(brlenRemoveAll);
+
   //Grafen method:
   QPushButton* brlenInitGrafen = new QPushButton(tr("Init"));
   connect(brlenInitGrafen, SIGNAL(clicked(bool)), this, SLOT(initLengthsGrafen()));
@@ -578,9 +600,37 @@ void PhyView::createBrlenPanel_()
   brlenLayout->addWidget(brlenToClockTree);
 
   //Midpoint rooting:
-  QPushButton* brlenMidpointRooting = new QPushButton(tr("Midpoint rooting"));
-  connect(brlenMidpointRooting, SIGNAL(clicked(bool)), this, SLOT(midpointRooting()));
-  brlenLayout->addWidget(brlenMidpointRooting);
+  brlenMidpointRootingCriteria_ =  new QComboBox;
+  brlenMidpointRootingCriteria_->addItem("Sum of squares");
+  brlenMidpointRootingCriteria_->addItem("Variance");
+  brlenMidpointRootingCriteria_->setEditable(false);
+  QPushButton* brlenMidpointRootingGo = new QPushButton(tr("Go!"));
+  connect(brlenMidpointRootingGo, SIGNAL(clicked(bool)), this, SLOT(midpointRooting()));
+  QGroupBox* brlenMidpointRootingBox = new QGroupBox(tr("Midpoint rooting"));
+  QHBoxLayout* brlenMidpointRootingLayout = new QHBoxLayout;
+  brlenMidpointRootingLayout->addWidget(brlenMidpointRootingCriteria_);
+  brlenMidpointRootingLayout->addWidget(brlenMidpointRootingGo);
+  brlenMidpointRootingLayout->addStretch(1);
+  brlenMidpointRootingBox->setLayout(brlenMidpointRootingLayout);
+
+  brlenLayout->addWidget(brlenMidpointRootingBox);
+
+  //Unresolved uncertain trees:
+  bootstrapThreshold_ = new QDoubleSpinBox;
+  bootstrapThreshold_->setValue(60);
+  bootstrapThreshold_->setDecimals(2);
+  bootstrapThreshold_->setSingleStep(0.1);
+  QPushButton* unresolveUncertainNodesGo = new QPushButton(tr("Go!"));
+  connect(unresolveUncertainNodesGo, SIGNAL(clicked(bool)), this, SLOT(unresolveUncertainNodes()));
+
+  QGroupBox* unresolveUncertainNodesBox = new QGroupBox(tr("Unresolve uncertain nodes"));
+  QHBoxLayout* unresolveUncertainNodesLayout = new QHBoxLayout;
+  unresolveUncertainNodesLayout->addWidget(bootstrapThreshold_);
+  unresolveUncertainNodesLayout->addWidget(unresolveUncertainNodesGo);
+  unresolveUncertainNodesLayout->addStretch(1);
+  unresolveUncertainNodesBox->setLayout(unresolveUncertainNodesLayout);
+  
+  brlenLayout->addWidget(unresolveUncertainNodesBox);
 
   ////
   brlenLayout->addStretch(1);
@@ -800,6 +850,7 @@ TreeDocument* PhyView::createNewDocument(Tree* tree)
   treeControlers_->applyOptions(&subWindow->getTreeCanvas());
   subWindow->show();
   setCurrentSubWindow(subWindow);
+  updateTreesTable();
   return doc;
 }
 
@@ -816,13 +867,22 @@ QList<TreeDocument*> PhyView::getNonActiveDocuments()
   return documents;
 }
 
+QList<TreeDocument*> PhyView::getDocuments()
+{
+  QList<TreeDocument*> documents;
+  QList<QMdiSubWindow *> lst = mdiArea_->subWindowList();
+  for (int i = 0; i < lst.size(); ++i) {
+    documents.push_back(dynamic_cast<TreeSubWindow*>(lst[i])->getDocument());
+  }
+  return documents;
+}
 
 
 void PhyView::readTree(const QString& path, const string& format)
 {
-  auto_ptr<ITree> treeReader(ioTreeFactory_.createReader(format));
+  unique_ptr<ITree> treeReader(ioTreeFactory_.createReader(format));
   try {
-    auto_ptr<Tree> tree(treeReader->read(path.toStdString()));
+    unique_ptr<Tree> tree(treeReader->read(path.toStdString()));
     TreeDocument* doc = createNewDocument(tree.get());
     doc->setFile(path.toStdString(), format);
     saveAction_->setEnabled(true);
@@ -841,6 +901,7 @@ void PhyView::readTree(const QString& path, const string& format)
     fileMenu_->insertAction(exitAction_, closeAction_);
     fileMenu_->insertAction(exitAction_, exportAction_);
     fileMenu_->insertAction(exitAction_, printAction_);
+    updateTreesTable();
   } catch (Exception& e) {
     QMessageBox::critical(this, tr("Ouch..."), tr("Error when reading file:\n") + tr(e.what()));
   }
@@ -874,6 +935,16 @@ void PhyView::setCurrentSubWindow(TreeSubWindow* tsw)
     treeControlers_->actualizeOptions();
     manager_.setActiveStack(&tsw->getDocument()->getUndoStack());
   }
+  //Update selection in tree table:
+  updateTreesTable(); //We need this here as some windows may have been closed.
+  QList<QMdiSubWindow *> lst = mdiArea_->subWindowList();
+  for (int i = 0; i < lst.size(); ++i) {
+    if (lst[i] == mdiArea_->activeSubWindow()) {
+      treesTable_->setRangeSelected(QTableWidgetSelectionRange(i, 0, i, 1), true);
+    } else {
+      treesTable_->setRangeSelected(QTableWidgetSelectionRange(i, 0, i, 1), false);
+    }
+  }
 }
 
 bool PhyView::saveTree()
@@ -882,7 +953,7 @@ bool PhyView::saveTree()
   if (doc->getFilePath() == "")
     return saveTreeAs();
   string format = doc->getFileFormat();
-  auto_ptr<OTree> treeWriter(ioTreeFactory_.createWriter(format));
+  unique_ptr<OTree> treeWriter(ioTreeFactory_.createWriter(format));
   Nhx* nhx = dynamic_cast<Nhx*>(treeWriter.get());
   if (nhx) {
     TreeTemplate<Node> treeCopy(*doc->getTree());
@@ -939,6 +1010,24 @@ void PhyView::closeTree()
     exportAction_->setDisabled(true);
     saveAction_->setDisabled(true);
   }
+  updateTreesTable();
+}
+
+void PhyView::updateTreesTable()
+{
+  //Update tree list:
+  treesTable_->clearSelection();
+  treesTable_->clearContents();
+  QList<QMdiSubWindow *> lst = mdiArea_->subWindowList();
+  treesTable_->setRowCount(lst.size());
+  for (int i = 0; i < lst.size(); ++i) {
+    TreeDocument* doc = dynamic_cast<TreeSubWindow*>(lst[i])->getDocument();
+    string docName = doc->getName();
+    if (docName == "")
+      docName = "Tree#" + TextTools::toString(i + 1); 
+    treesTable_->setItem(i, 0, new QTableWidgetItem(QtTools::toQt(docName)));
+    treesTable_->setItem(i, 1, new QTableWidgetItem(QtTools::toQt(TextTools::toString<unsigned int>(doc->getTree()->getNumberOfLeaves()))));
+  }
 }
 
 void PhyView::exit()
@@ -949,16 +1038,16 @@ void PhyView::exit()
 void PhyView::aboutBpp()
 {
   QMessageBox msgBox;
-  msgBox.setText("Bio++ 2.1.0.");
-  msgBox.setInformativeText("bpp-core 2.1.0\nbpp-seq 2.1.0.\nbpp-phyl 2.1.0.\nbpp-qt 2.1.0");
+  msgBox.setText("Bio++ 2.4.0.");
+  msgBox.setInformativeText("bpp-core 2.4.0\nbpp-seq 2.4.0.\nbpp-phyl 2.4.0.\nbpp-qt 2.4.0");
   msgBox.exec();
 }
 
 void PhyView::about()
 {
   QMessageBox msgBox;
-  msgBox.setText("This is Bio++ Phy View version 0.3.0.");
-  msgBox.setInformativeText("Julien Dutheil <julien.dutheil@univ-montp2.fr>.");
+  msgBox.setText("This is Bio++ Phylogenetic Viewer version 0.6.0.");
+  msgBox.setInformativeText("Julien Dutheil <dutheil@evolbio.mpg.de>.");
   msgBox.exec();
 }
 
@@ -994,12 +1083,30 @@ void PhyView::midpointRooting()
 {
   if (hasActiveDocument())
     try {
-      submitCommand(new MidpointRootingCommand(getActiveDocument()));
+      submitCommand(new MidpointRootingCommand(getActiveDocument(), brlenMidpointRootingCriteria_->currentText().toStdString()));
     } catch (NodeException& ex) {
       QMessageBox::critical(this, tr("Oups..."), tr("Some branch do not have lengths."));
     }
 }
 
+void PhyView::deleteAllLengths()
+{
+  if (hasActiveDocument())
+    submitCommand(new DeleteLengthCommand(getActiveDocument()));
+}
+
+
+void PhyView::unresolveUncertainNodes()
+{
+  if (hasActiveDocument()) {
+    try {
+      submitCommand(new UnresolveUnsupportedNodesCommand(getActiveDocument(), bootstrapThreshold_->value()));
+    } catch (NodeException& ex) {
+      QMessageBox::critical(this, tr("Oups..."), tr("An exception occurred while unresolving your tree!"));
+    }
+  }
+}
+
 void PhyView::translateNames()
 {
   if (hasActiveDocument())
@@ -1139,9 +1246,80 @@ void PhyView::searchResultSelected()
   getActiveSubWindow()->getTreeCanvas().ensureVisible(searchResultsItems_[searchResults_->currentRow()]);
 }
 
+void PhyView::activateSelectedDocument() {
+  if (treesTable_->selectedItems().size() > 0) {
+    int index = treesTable_->selectedItems()[0]->row();
+    mdiArea_->setActiveSubWindow(mdiArea_->subWindowList()[index]);
+    mdiArea_->activeSubWindow()->showNormal();
+    mdiArea_->activeSubWindow()->raise();
+  }
+}
+
+
+TreeTemplate<Node>* PhyView::pickTree()
+{
+  QList<TreeDocument*> documents = getDocuments();
+  //treeList_->clear();
+  QStringList items;
+  for (int i = 0; i < documents.size(); ++i) {
+    QString text = QtTools::toQt(documents[i]->getName());
+    if (text == "") text = "(unknown)";
+    vector<string> leaves = documents[i]->getTree()->getLeavesNames(); 
+    text += QtTools::toQt(" " + TextTools::toString(leaves.size()) + " leaves ");
+
+    for (unsigned int j = 0; j < min(static_cast<unsigned int>(leaves.size()), 5u); ++j) {
+      text += QtTools::toQt(", " + leaves[j]);
+    }
+    if (leaves.size() >= 5) text += "...";
+    //treeList_->addItem(text);
+    items << text;
+  }
+
+  //treeChooser_->exec();
+  //int index = treeList_->currentRow();
+  //return index > 0 ? documents[index]->getTree() : 0;
+  bool ok;
+  QString item = QInputDialog::getItem(this, "Pick a tree", "Tree to insert:", items, 0, false, &ok);
+  if (ok && !item.isEmpty())
+    return documents[items.indexOf(item)]->getTree();
+  else
+    return 0;
+}
+
+
+
+
+
+// This class is necessary to reimplement the notify method, in order to catch any foreign exception.
+class PhyViewApplication:
+  public QApplication
+{
+  public:
+    PhyViewApplication(int &argc, char *argv[]):
+      QApplication(argc, argv) {}
+
+  public:
+    bool notify(QObject *receiver_, QEvent *event_)
+    {
+      try
+      {
+        return QApplication::notify(receiver_, event_);
+      }
+      catch (std::exception &ex)
+      {
+        std::cerr << "std::exception was caught" << std::endl;
+        std::cerr << ex.what() << endl;
+        QMessageBox msgBox;
+        msgBox.setText(ex.what());
+        msgBox.exec();
+      }
+      return false;
+    }
+};
+
 int main(int argc, char *argv[])
 {
-  QApplication app(argc, argv);
+  PhyViewApplication app(argc, argv);
 
   PhyView* phyview = new PhyView();
   phyview->show();
@@ -1149,7 +1327,7 @@ int main(int argc, char *argv[])
   //Parse command line arguments:
   QStringList args = app.arguments();
   string format = IOTreeFactory::NEWICK_FORMAT;
-  QTextCodec* codec = QTextCodec::codecForLocale();
+  //QTextCodec* codec = QTextCodec::codecForLocale(); Not supported in Qt5...
   for (int i = 1; i < args.size(); ++i) {
     if (args[i] == "--nhx") {
       format = IOTreeFactory::NHX_FORMAT;
@@ -1157,18 +1335,17 @@ int main(int argc, char *argv[])
       format = IOTreeFactory::NEWICK_FORMAT;
     } else if (args[i] == "--newick") {
       format = IOTreeFactory::NEWICK_FORMAT;
-    } else if (args[i] == "--enc") {
-      if (i == args.size() - 1) {
-        cerr << "You must specify a text encoding after --enc tag." << endl;
-        exit(1);
-      }
-      ++i;
-      codec = QTextCodec::codecForName(args[i].toStdString().c_str());
+    //} else if (args[i] == "--enc") {
+    //  if (i == args.size() - 1) {
+    //    cerr << "You must specify a text encoding after --enc tag." << endl;
+    //    exit(1);
+    //  }
+    //  ++i;
+    //  codec = QTextCodec::codecForName(args[i].toStdString().c_str());
     } else {
       phyview->readTree(args[i], format);
     }
   }
-  QTextCodec::setCodecForCStrings(codec);
   return app.exec();
 }
 
